/*******************************************************************************
* File Name: CNTR_IR_Capture.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "CNTR_IR_Capture.h"

uint8 CNTR_IR_Capture_initVar = 0u;


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_Init(void) 
{
        #if (!CNTR_IR_Capture_UsingFixedFunction && !CNTR_IR_Capture_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!CNTR_IR_Capture_UsingFixedFunction && !CNTR_IR_Capture_ControlRegRemoved) */
        
        #if(!CNTR_IR_Capture_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 CNTR_IR_Capture_interruptState;
        #endif /* (!CNTR_IR_Capture_UsingFixedFunction) */
        
        #if (CNTR_IR_Capture_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            CNTR_IR_Capture_CONTROL &= CNTR_IR_Capture_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                CNTR_IR_Capture_CONTROL2 &= ((uint8)(~CNTR_IR_Capture_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                CNTR_IR_Capture_CONTROL3 &= ((uint8)(~CNTR_IR_Capture_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (CNTR_IR_Capture_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                CNTR_IR_Capture_CONTROL |= CNTR_IR_Capture_ONESHOT;
            #endif /* (CNTR_IR_Capture_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            CNTR_IR_Capture_CONTROL2 |= CNTR_IR_Capture_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            CNTR_IR_Capture_RT1 &= ((uint8)(~CNTR_IR_Capture_RT1_MASK));
            CNTR_IR_Capture_RT1 |= CNTR_IR_Capture_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            CNTR_IR_Capture_RT1 &= ((uint8)(~CNTR_IR_Capture_SYNCDSI_MASK));
            CNTR_IR_Capture_RT1 |= CNTR_IR_Capture_SYNCDSI_EN;

        #else
            #if(!CNTR_IR_Capture_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = CNTR_IR_Capture_CONTROL & ((uint8)(~CNTR_IR_Capture_CTRL_CMPMODE_MASK));
            CNTR_IR_Capture_CONTROL = ctrl | CNTR_IR_Capture_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = CNTR_IR_Capture_CONTROL & ((uint8)(~CNTR_IR_Capture_CTRL_CAPMODE_MASK));
            
            #if( 0 != CNTR_IR_Capture_CAPTURE_MODE_CONF)
                CNTR_IR_Capture_CONTROL = ctrl | CNTR_IR_Capture_DEFAULT_CAPTURE_MODE;
            #else
                CNTR_IR_Capture_CONTROL = ctrl;
            #endif /* 0 != CNTR_IR_Capture_CAPTURE_MODE */ 
            
            #endif /* (!CNTR_IR_Capture_ControlRegRemoved) */
        #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!CNTR_IR_Capture_UsingFixedFunction)
            CNTR_IR_Capture_ClearFIFO();
        #endif /* (!CNTR_IR_Capture_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        CNTR_IR_Capture_WritePeriod(CNTR_IR_Capture_INIT_PERIOD_VALUE);
        #if (!(CNTR_IR_Capture_UsingFixedFunction && (CY_PSOC5A)))
            CNTR_IR_Capture_WriteCounter(CNTR_IR_Capture_INIT_COUNTER_VALUE);
        #endif /* (!(CNTR_IR_Capture_UsingFixedFunction && (CY_PSOC5A))) */
        CNTR_IR_Capture_SetInterruptMode(CNTR_IR_Capture_INIT_INTERRUPTS_MASK);
        
        #if (!CNTR_IR_Capture_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)CNTR_IR_Capture_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            CNTR_IR_Capture_WriteCompare(CNTR_IR_Capture_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            CNTR_IR_Capture_interruptState = CyEnterCriticalSection();
            
            CNTR_IR_Capture_STATUS_AUX_CTRL |= CNTR_IR_Capture_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(CNTR_IR_Capture_interruptState);
            
        #endif /* (!CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void CNTR_IR_Capture_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (CNTR_IR_Capture_UsingFixedFunction)
        CNTR_IR_Capture_GLOBAL_ENABLE |= CNTR_IR_Capture_BLOCK_EN_MASK;
        CNTR_IR_Capture_GLOBAL_STBY_ENABLE |= CNTR_IR_Capture_BLOCK_STBY_EN_MASK;
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!CNTR_IR_Capture_ControlRegRemoved || CNTR_IR_Capture_UsingFixedFunction)
        CNTR_IR_Capture_CONTROL |= CNTR_IR_Capture_CTRL_ENABLE;                
    #endif /* (!CNTR_IR_Capture_ControlRegRemoved || CNTR_IR_Capture_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  CNTR_IR_Capture_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void CNTR_IR_Capture_Start(void) 
{
    if(CNTR_IR_Capture_initVar == 0u)
    {
        CNTR_IR_Capture_Init();
        
        CNTR_IR_Capture_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    CNTR_IR_Capture_Enable();        
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void CNTR_IR_Capture_Stop(void) 
{
    /* Disable Counter */
    #if(!CNTR_IR_Capture_ControlRegRemoved || CNTR_IR_Capture_UsingFixedFunction)
        CNTR_IR_Capture_CONTROL &= ((uint8)(~CNTR_IR_Capture_CTRL_ENABLE));        
    #endif /* (!CNTR_IR_Capture_ControlRegRemoved || CNTR_IR_Capture_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (CNTR_IR_Capture_UsingFixedFunction)
        CNTR_IR_Capture_GLOBAL_ENABLE &= ((uint8)(~CNTR_IR_Capture_BLOCK_EN_MASK));
        CNTR_IR_Capture_GLOBAL_STBY_ENABLE &= ((uint8)(~CNTR_IR_Capture_BLOCK_STBY_EN_MASK));
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_SetInterruptMode(uint8 interruptsMask) 
{
    CNTR_IR_Capture_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   CNTR_IR_Capture_ReadStatusRegister(void) 
{
    return CNTR_IR_Capture_STATUS;
}


#if(!CNTR_IR_Capture_ControlRegRemoved)
/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   CNTR_IR_Capture_ReadControlRegister(void) 
{
    return CNTR_IR_Capture_CONTROL;
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    CNTR_IR_Capture_WriteControlRegister(uint8 control) 
{
    CNTR_IR_Capture_CONTROL = control;
}

#endif  /* (!CNTR_IR_Capture_ControlRegRemoved) */


#if (!(CNTR_IR_Capture_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: CNTR_IR_Capture_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void CNTR_IR_Capture_WriteCounter(uint32 counter) \
                                   
{
    #if(CNTR_IR_Capture_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (CNTR_IR_Capture_GLOBAL_ENABLE & CNTR_IR_Capture_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        CNTR_IR_Capture_GLOBAL_ENABLE |= CNTR_IR_Capture_BLOCK_EN_MASK;
        CY_SET_REG16(CNTR_IR_Capture_COUNTER_LSB_PTR, (uint16)counter);
        CNTR_IR_Capture_GLOBAL_ENABLE &= ((uint8)(~CNTR_IR_Capture_BLOCK_EN_MASK));
    #else
        CY_SET_REG32(CNTR_IR_Capture_COUNTER_LSB_PTR, counter);
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}
#endif /* (!(CNTR_IR_Capture_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
*******************************************************************************/
uint32 CNTR_IR_Capture_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(CNTR_IR_Capture_UsingFixedFunction)
		(void)CY_GET_REG16(CNTR_IR_Capture_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(CNTR_IR_Capture_COUNTER_LSB_PTR_8BIT);
	#endif/* (CNTR_IR_Capture_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(CNTR_IR_Capture_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(CNTR_IR_Capture_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG32(CNTR_IR_Capture_STATICCOUNT_LSB_PTR));
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
*******************************************************************************/
uint32 CNTR_IR_Capture_ReadCapture(void) 
{
    #if(CNTR_IR_Capture_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(CNTR_IR_Capture_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG32(CNTR_IR_Capture_STATICCOUNT_LSB_PTR));
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_WritePeriod(uint32 period) 
{
    #if(CNTR_IR_Capture_UsingFixedFunction)
        CY_SET_REG16(CNTR_IR_Capture_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG32(CNTR_IR_Capture_PERIOD_LSB_PTR, period);
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
*******************************************************************************/
uint32 CNTR_IR_Capture_ReadPeriod(void) 
{
    #if(CNTR_IR_Capture_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(CNTR_IR_Capture_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG32(CNTR_IR_Capture_PERIOD_LSB_PTR));
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


#if (!CNTR_IR_Capture_UsingFixedFunction)
/*******************************************************************************
* Function Name: CNTR_IR_Capture_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_WriteCompare(uint32 compare) \
                                   
{
    #if(CNTR_IR_Capture_UsingFixedFunction)
        CY_SET_REG16(CNTR_IR_Capture_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG32(CNTR_IR_Capture_COMPARE_LSB_PTR, compare);
    #endif /* (CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
*******************************************************************************/
uint32 CNTR_IR_Capture_ReadCompare(void) 
{
    return (CY_GET_REG32(CNTR_IR_Capture_COMPARE_LSB_PTR));
}


#if (CNTR_IR_Capture_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: CNTR_IR_Capture_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    CNTR_IR_Capture_CONTROL &= ((uint8)(~CNTR_IR_Capture_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    CNTR_IR_Capture_CONTROL |= compareMode;
}
#endif  /* (CNTR_IR_Capture_COMPARE_MODE_SOFTWARE) */


#if (CNTR_IR_Capture_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: CNTR_IR_Capture_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void CNTR_IR_Capture_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    CNTR_IR_Capture_CONTROL &= ((uint8)(~CNTR_IR_Capture_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    CNTR_IR_Capture_CONTROL |= ((uint8)((uint8)captureMode << CNTR_IR_Capture_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (CNTR_IR_Capture_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: CNTR_IR_Capture_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void CNTR_IR_Capture_ClearFIFO(void) 
{

    while(0u != (CNTR_IR_Capture_ReadStatusRegister() & CNTR_IR_Capture_STATUS_FIFONEMP))
    {
        (void)CNTR_IR_Capture_ReadCapture();
    }

}
#endif  /* (!CNTR_IR_Capture_UsingFixedFunction) */


/* [] END OF FILE */

