/*******************************************************************************
* File Name: cr_IR_OUT_Level_PM.c
* Version 1.80
*
* Description:
*  This file contains the setup, control, and status commands to support 
*  the component operation in the low power mode. 
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cr_IR_OUT_Level.h"

/* Check for removal by optimization */
#if !defined(cr_IR_OUT_Level_Sync_ctrl_reg__REMOVED)

static cr_IR_OUT_Level_BACKUP_STRUCT  cr_IR_OUT_Level_backup = {0u};

    
/*******************************************************************************
* Function Name: cr_IR_OUT_Level_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void cr_IR_OUT_Level_SaveConfig(void) 
{
    cr_IR_OUT_Level_backup.controlState = cr_IR_OUT_Level_Control;
}


/*******************************************************************************
* Function Name: cr_IR_OUT_Level_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*
*******************************************************************************/
void cr_IR_OUT_Level_RestoreConfig(void) 
{
     cr_IR_OUT_Level_Control = cr_IR_OUT_Level_backup.controlState;
}


/*******************************************************************************
* Function Name: cr_IR_OUT_Level_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component for entering the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void cr_IR_OUT_Level_Sleep(void) 
{
    cr_IR_OUT_Level_SaveConfig();
}


/*******************************************************************************
* Function Name: cr_IR_OUT_Level_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component after waking up from the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void cr_IR_OUT_Level_Wakeup(void)  
{
    cr_IR_OUT_Level_RestoreConfig();
}

#endif /* End check for removal by optimization */


/* [] END OF FILE */
