/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*
    This algorithm started life as a guess.
    It is improved through empirical analysis.
    What you see may or may not be what was
    originally conceived.

 * ========================================
*/
#include "main.h"

float temperature=70.0;// this is set elsewhere
int16_t NumberOfTemperatureCommandsToSend=25;

//float upperTemperature,lowerTemperature;
float temperatureRange,temperatureOffset;
int32 decisionTimer;
uint16 lastMinutes;
int16_t TemperatureReadingAvailable;

enum ACControlStates {
    StateTemperatureStart=0
    ,StateTemperatureTakeReading
    ,StateTemperatureTooHigh
    ,StateTemperatureJustRight
    ,StateTemperatureTooLow
    ,StateTemperatureTestReading
    ,StateTemperatureWaitForTimeout
};

enum ACControlStates ACControlState,ACLastControlState=StateTemperatureJustRight;

//-----------------------------------------------------------------------------
// update the state machine's
// state based on current temperature
//-----------------------------------------------------------------------------
enum ACControlStates processCurrentTemperature(uint16 selectedTemperature){
  enum ACControlStates stateToRun = StateTemperatureTakeReading;
    
    if ( temperature < selectedTemperature -temperatureRange ){
        stateToRun = StateTemperatureTooLow;
    } else {
        if (temperature > selectedTemperature + temperatureRange){
            stateToRun=StateTemperatureTooHigh;
        } else {
            stateToRun = StateTemperatureJustRight;
        }
    }
    
    return stateToRun;
}
//-----------------------------------------------------------------------------
// initialize for operation
//-----------------------------------------------------------------------------
void init_TemperatureControl(){
    
    ACControlState=StateTemperatureStart;
    temperatureOffset= 0.5f;
    temperatureRange = 0.5f; // 0.5 makes 71 setting toggle at 70 or 72 due to offset.   
    debugPrint=YES;// during debug
    sendCountDownTextToNextion(decisionTimer);
    do {
        temperature=getTemperatureF(&TemperatureReadingAvailable);
    } while (!TemperatureReadingAvailable);
    
    extern void sendTemperatureToNextion();
    sendTemperatureToNextion();
}

//-----------------------------------------------------------------------------
// send temperature down command
//-----------------------------------------------------------------------------
void sendIRSequence(enum captureTypesEnum cType,enum equipEnum equipID){
        addPlayIRToQueue(cType,equipID);
}
//-----------------------------------------------------------------------------
// send temperature down commands
// # to send was determined after testing
//-----------------------------------------------------------------------------
void sendTempDown() {
    if (debugPrint)
    printMessage("Send Temp DOWN.\n\r");

int16 i;
    for (i=0; i < NumberOfTemperatureCommandsToSend;i++)
        sendIRSequence(downType,CurrentEquipmentID);
}
//-----------------------------------------------------------------------------
// send 3 temperature up commands
// this was determined empirically
//-----------------------------------------------------------------------------
void sendTempUp() {
    if (debugPrint)
    printMessage("Send Temp UP.\n\r");

int16 i;
    for (i=0; i < NumberOfTemperatureCommandsToSend;i++)
        sendIRSequence(upType,CurrentEquipmentID);
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void  printCurrentState(){
    
    
}

int16 downPulseCount;
int16 upPulseCount;
int16 targetReachStoppedDone;
//-----------------------------------------------------------------------------
// todo: need to add delta temperature and
// microphone to listen to the aircon.
//------------------------------------
//  if we are in Manual Mode then exit
//
//  if we are in the heat/cool mode then ---
//      if the temperature is too low then ---
//          if we have not changed any setting in 
//               the last 5 minutes then ---
//                      increment the temperature by 3 pulses
//                          UNLESS we already have gone up 12 pulses
//          endif ...
//      endif ...
//  else --- we are in heat/cool mode
//      if the temperature is too high then ---
//          if we have not changed any setting in 
//              the last 5 minutes then ---
//                     decrement the temperature by 3 pulses
//                          UNLESS we have already down by 12 pulses
//          endif ...
//      endif ....
//  endif ...
//
//    
//-----------------------------------------------------------------------------
void handle_TemperatureControl(){
            
    if (manualMode)
        return;
    
    switch(ACControlState) {
        case StateTemperatureStart:{
                ACControlState=StateTemperatureTakeReading;
        }break;
        
        case StateTemperatureTakeReading:{
            temperature=getTemperatureF(&TemperatureReadingAvailable);
            if (!TemperatureReadingAvailable)
                break;

            if (decisionTimer>0)
                ACControlState = StateTemperatureWaitForTimeout;            
            else
                ACControlState = StateTemperatureTestReading;
        }break;
            
        case StateTemperatureTestReading:{
            if (debugPrint){
                printMessage("Testing Temperature\n\r");
                sendDebugTextToNextion("Testing");
            }
            float testTemp;
            if (operatingMode==COOL)
                testTemp = coolTemp;
            else
                testTemp = heatTemp;
            
            
            if (temperature < testTemp+temperatureOffset-temperatureRange){                        
                ACControlState= StateTemperatureTooLow;
                break;
            } 
            
            if (temperature > testTemp+temperatureOffset+temperatureRange){
                ACControlState = StateTemperatureTooHigh;
                break;
            }
            
            ACControlState = StateTemperatureJustRight;
        }break;   
                   
        case StateTemperatureTooHigh:{
            if (debugPrint){
                printMessage("Temperature too high.\n\r");
            }
                
            sendDebugTextToNextion("Temp Too High");
            // if we are GE, it uses a round robin, will have to handle that elsewhere
            if (CurrentEquipmentID == FRIDGIDAIRE_ID){
                if (debugPrint) printMessage("F:");
              
                #if 0
                // play the heat/cool setting command
                // only play the heat/cool type once, if we are currently 
                // frigidaire, as it has those selects
                if (operatingMode==HEAT)
                    addPlayIRToQueue(heatType,CurrentEquipmentID);
                else
                    addPlayIRToQueue(coolType,CurrentEquipmentID);
                #endif
            }else {
               if (debugPrint) printMessage("GE:");
             }
            
            // the following code will play the adjustment to the AC unit
            // if this is the first time since temp was just right, move
            // the A/C unit hard the other direction, else just a single tap
            if (ACLastControlState == StateTemperatureJustRight)
                NumberOfTemperatureCommandsToSend = 25;
            else
                NumberOfTemperatureCommandsToSend =1;
            
            ACLastControlState = ACControlState;
            
            sendTempDown();
            decisionTimer=4;// wait 4 minutes
            ACControlState = StateTemperatureWaitForTimeout;
            sendCountDownTextToNextion(decisionTimer);
        }break;
        
        case StateTemperatureJustRight:{
            ACLastControlState = ACControlState;
           if (debugPrint){
                printMessage("Temperature Good.\n\r");
            }
                sendDebugTextToNextion("Temp OK");
            decisionTimer=1; // wait 1 minute and check again 
            sendCountDownTextToNextion(decisionTimer);
            ACControlState = StateTemperatureWaitForTimeout; 
        }break;
        
        case StateTemperatureTooLow:{
            if (debugPrint){
                printMessage("Temperature too Low.\n\r");
            }

            sendDebugTextToNextion("Temp Too Low");
             // only play the heat/cool type once, if we are currently 
            // frigidaire, as it has those selects
            // if we are GE, it uses a round robin, will have to handle that elsewhere
            if (CurrentEquipmentID == FRIDGIDAIRE_ID){
                if (debugPrint) printMessage("F:");
                
                #if 0
                // play the heat/cool setting command
                if (operatingMode==HEAT)
                    addPlayIRToQueue(heatType,CurrentEquipmentID);
                else
                    addPlayIRToQueue(coolType,CurrentEquipmentID);
                #endif
            }else {
               if (debugPrint) printMessage("GE:");
             }
            
            // the following code will play the adjustment to the AC unit
            // if this is the first time since temp was just right, move
            // the A/C unit hard the other direction, else just a single tap
            if (ACLastControlState == StateTemperatureJustRight)
                NumberOfTemperatureCommandsToSend = 25;
            else
                NumberOfTemperatureCommandsToSend =1;
            
            ACLastControlState = ACControlState;
            
            sendTempUp();
            decisionTimer=4;// wait 4 minutes
            ACControlState = StateTemperatureWaitForTimeout;
            sendCountDownTextToNextion(decisionTimer);
        }break;
        
       
        case StateTemperatureWaitForTimeout:{
            // take temperature every 5 seconds
            if (msCounter1<=0){
                ACControlState=StateTemperatureTakeReading;
                msCounter1=5000; // one time per 5 seconds
                break;
            }
            if (minutes != lastMinutes){
                lastMinutes = minutes;
                if (decisionTimer>0){
              
                    decisionTimer--;
                    sendCountDownTextToNextion(decisionTimer);

                    if (debugPrint){
                        printMessage(": decisionTimer decrement\n\r");
                    }

                }
            }
             
            
            if (decisionTimer>0)
                break;
            
            ACControlState = StateTemperatureTestReading;
        }break;
    }
   
}



/* [] END OF FILE */
