/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
#include "project.h"
#include <stdlib.h>
#include "USBSerial.h"

#define USBFS_DEVICE (0)  // not sure why needed

    
USBSerialMessage USBRxBuffer[NUMBER_OF_USB_BUFFERS+1];

int16_t currentBuffer;

/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/
int16 usbserial_putString(const char msg[])
{
    int16 holdCount=0;
    if(0 == USBUART_GetConfiguration()) 
        return -1;
    if (0==USBUART_CheckActivity())
        return -1;
    
  
    
        // if no answer after 25 milliseconds, return

    while(0 == USBUART_CDCIsReady()){
        CyDelay(1); // wait 1 millisecond
        // then bail
        if (holdCount++> 24)
            return -1; 
    }
        
    holdCount=0;
    while(0 == USBUART_CDCIsReady()){
        CyDelay(1); // wait 1 millisecond
        // if no answer after 25 milliseconds, 
        // then bail
        if (holdCount++> 24)
            return -1; 
    }

    if (USBUART_CDCIsReady())
        USBUART_PutString(msg);
        
    CyDelay(1);// wait for uart to clear
    return 1;
}
/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/
int16 usbserial_putChar(char c) {
    static char cToPrint[2];
    
    cToPrint[0]=c;
    cToPrint[1]=0;
    return usbserial_putString(cToPrint);
}

    uint16_t rxCount;
    uint16_t bufferNumber=0;
/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/
void initUSBSerial( )
{
    
    //int index;
    

    /* Start the USB_UART */
    /* Start USBFS operation with 5-V operation. */
    USBUART_Start(USBFS_DEVICE, USBUART_5V_OPERATION);
}
/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/
int16 handleUSBSerial() {
    int16 bufNum;
    
        /* Host can send double SET_INTERFACE request. */
        if (0u != USBUART_IsConfigurationChanged())
        {
            /* Initialize IN endpoints when device is configured. */
            if (0u != USBUART_GetConfiguration())
            {
                /* Enumeration is done, enable OUT endpoint to receive data 
                 * from host. */
                USBUART_CDC_Init();
            }
        }
        
        if(0 != USBUART_GetConfiguration())
        {   
            /* Check for input data from host. */
            if (0u != USBUART_DataIsReady()){
                USBSerialMessage *ptr = &USBRxBuffer[bufferNumber];
                /* Read received data and re-enable OUT endpoint. */
                rxCount = USBUART_GetAll(ptr->msg);
                // now post these bytes to the Debug Terminal Task
                ptr->ucMessageID = bufNum=bufferNumber;
                ptr->size=rxCount;
                ptr->msg[rxCount]=0;// terminate string just in case
                
                // send received message to debug terminal task.
                // Future: we can route messages anywhere depending upon
                // message contents.
                bufferNumber++;
                if (bufferNumber >= NUMBER_OF_USB_BUFFERS)
                    bufferNumber=0; // reset the ring
                
                return bufNum;
            }
        }

    return -1;
}

// end of file
