/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
#include "atoi_itoa.h"


//-------------------------------------------
//-------------------------------------------
//only pos values are correct
//returns -1 on error
//-------------------------------------------
//-------------------------------------------
int32 myStrToUint(char *src)
{
    char tChar   = *src;
    int  convInt = 0;
    
    //look for sign, error if present
    if(tChar == '-')
    {
        return -1;
    }
    // for us, any character other than a number will stop parse
    while(tChar >= 0x30 && tChar<=0x39)
    {
        //only want numbers, error on all others
        if( (tChar > 0x2F) && (tChar < 0x3A) )
        {
            tChar -= 0x30;
            convInt = (convInt*10) + tChar;
        }
        else
        {
            return -1;
        }
        src++;
        tChar = *src;
    }
    return convInt;
}

//==============================================
// ascii helper below
//-------------------------------------------
//-------------------------------------------
int16 _digit(uint8 c) {
    if ( c < '0' || c > '9')
        return 0;
    
    return 1;
}
// ==========================================================
// some of this code (i2a) originally obtained from gforge,
// modified by wsm, original had the following license:
// Utility functions
//
// Design and implementation by
// - Floris van den Berg (flvdberg@wxs.nl)
// - Hervé Drolon <drolon@infonie.fr>
// - Ryan Rubley (ryan@lostreality.org)
//
// This file is part of FreeImage 3
//
// COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
// THAT THE COVERED CODE IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE
// OR NON-INFRINGING. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE COVERED
// CODE IS WITH YOU. SHOULD ANY COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT
// THE INITIAL DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
// SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL
// PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
// THIS DISCLAIMER.
//
// Use at your own risk!
// ==========================================================


//-------------------------------------------
 /*
 ** The following two functions together make up an itoa()
 ** implementation. Function i2a() is a 'private' function
 ** called by the public itoa() function.
 **
 ** itoa() takes three arguments:
 ** 1) the integer to be converted,
 ** 2) a pointer to a character conversion buffer,
 ** 3) the radix for the conversion
 ** which can range between 2 and 36 inclusive
 ** range errors on the radix default it to base10
 */

//-------------------------------------------
//-------------------------------------------
char *i2a(unsigned i, char *a, unsigned r)
{
	if (i/r > 0) a = i2a(i/r,a,r);
	*a = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i%r];
	return a+1;
}

//-------------------------------------------
// wsm return signed
//-------------------------------------------
char *itoa_s(int i, char *a, int r)
{
	if ((r < 2) || (r > 36)) r = 10;
	if (i<0) {
		*a = '-';
		*i2a(-(unsigned)i,a+1,r) = 0;
	} else *i2a(i,a,r) = 0;
	return a;
}

//-------------------------------------------
// wsm return unsigned
//-------------------------------------------
char *itoa_u(unsigned int i, char *a, int r)
{
	if ((r < 2) || (r > 36)) r = 10;
	*i2a(i,a,r) = 0;
	return a;
}


//---------------------------------------------
// return asci to integer, into minimim string
//---------------------------------------------
int atoi_n(const char * string, int n, int len, int *value_return)
{
    int i = n;
    int val = 0;
    int neg =0;
     
    if(i >= len )
        return -1;

    uint8 c=string[i];
      
    if (c=='-') {
        i++;
        neg=1;
        c=string[i];
    }
    
    if (!_digit(c))
        return -1;
    
    
    while(i < len && _digit(string[i])) {
        val = val * 10 + (string[i] - '0');
        i++;
    }
    if (neg)
       val = -val;
    *value_return = val;
    return i;
}

/* [] END OF FILE */
