/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
//#include "captureIR.h"
#include "main.h"

int16 captureIRData;

#if ENABLE_RECORDING
    
//--------------------------------------------------
//#define MAX_NUMBER_OF_TRANSITIONS_IN_ONE_IR_MESSAGE 256
//--------------------------------------------------
uint32      CaptureTimes[MAX_NUMBER_OF_TRANSITIONS_IN_ONE_IR_MESSAGE];
uint8       capCounter;
IR_CAPTURE  CapturedIR;                      // must be set to 0 initally
uint16      SendTimes[MAX_NUMBER_OF_TRANSITIONS_IN_ONE_IR_MESSAGE];


//--------------------------------------------------
//--------------------------------------------------
CY_ISR(CaptureISR){
    
    P1_5_CaptureISR_Orange_Write(!P1_5_CaptureISR_Orange_Read());// toggle the isr flag
    
    int16_t regVal=CNTR_IR_Capture_ReadStatusRegister();
    
     if (regVal & CNTR_IR_Capture_STATUS_OVERFLOW){
        capCounter=0;
        return;
    }
 
    if (regVal & CNTR_IR_Capture_STATUS_CAPTURE && regVal &CNTR_IR_Capture_STATUS_FIFONEMP)
       CaptureTimes[capCounter++]= CNTR_IR_Capture_ReadCapture()>>1;// divide by 2
    
    int16_t counter=4;
    while (counter-- && CNTR_IR_Capture_ReadStatusRegister() &CNTR_IR_Capture_STATUS_FIFONEMP)
     CNTR_IR_Capture_ReadCapture(); 

}


#endif

//--------------------------------------------------
// we have to store this in memory, and then
// save it into eeprom.
//--------------------------------------------------
void updateIRCaptureConfig() {
    
    int16 index;
    uint16 result;
  
    
    IR_CAPTURE *ircPtr;
    ircPtr = &CapturedIR;
    
    for (index =1; index < capCounter; index++){
        result = CaptureTimes[index]-CaptureTimes[index-1];
        ircPtr->TransitionTimes[index-1]=result;
    }    
    ircPtr->CaptureType =captureType;
    ircPtr->NumberOfTransitions = capCounter;
}
//--------------------------------------------------
//--------------------------------------------------
void init_capture(){
#if ENABLE_RECORDING
    isr_Capture_StartEx(CaptureISR);
    
    CNTR_IR_Capture_Start();
    
#endif
}
//--------------------------------------------------
//--------------------------------------------------
void handle_capture() {
 
    switch (captureIRData){
        case 0: case 2:
        break;
        
        case 1:
            CNTR_IR_Capture_Stop();
            CNTR_IR_Capture_WriteCounter(0);
            CNTR_IR_Capture_Enable();
            capCounter=0;
            captureIRData=2;
        break;
            
        case 3:
            CNTR_IR_Capture_Stop();
            captureIRData=0;
            // write the captured information to 
            // the config structure
            updateIRCaptureConfig();
        break;
    }
    
}

/* [] END OF FILE */
