/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
// ALL "BINARY" or HEXADECIMAL values are BIG ENDIAN

// information:
//  a # in a string is to be replaced by the number of the item you are working with
// example: "set b#" --> change to "set b0", where both the b and the 0 are human readable ascii.

#define END_OF_COMMAND  0xffffff  // three bytes to be sent (NOT 4 bytes!!!) at end of cmd

#define RESET_NEXTION "rest"
#define CLEAR_SCREEN_EX "cls #"  //clear to color
#define CLEAR_SCREEN_NAME "cls str"  //clear to color cls RED




#define REFRESH_PAGE "page #"
#define REFRESH_PAGE_MAIN "page main"

#define REFRESH_COMPONENT "ref "   // ie: "ref t0" to refresh component t0
#define REFRESH_COMPONENT_EX "ref @"  // the @ is the component id
//1. The default loading mode is automatically load when you create and edit 
//a component in Nextion Editor. If set it as manually load, you should use 
//ref command to load the component. Or when the component is covered by the 
//other components, you can use this command to refresh the covered component.

//2. Component auto refresh when attribute changes only valid for those 
//attributes in green bold font. The other attributes not in this format 
//can only be refreshed and displayed by the command of ref.

//3.When creating a component in Nextion editor, the default loading mode is 
//auto loading. When the loading mode is set as manual loading, it requires to 
//use ref to load. When the component is sheltered by GUI command mapping, or 
//when the component is sheltered by other manual loading components, you can 
//use ref to refresh the component. 

#define ACTIVATE_CLICK "click "
#define ACTIVATE_CLICK_EX "click @, #"  // click b0,0 == release, click b0,1 == press

#define REFRESH_STOP  "ref_stop" // stop refreshing screen

#define REFRESH_RESUME "ref_star" // start refreshing (use after ref_stop)

#define GET_VARIABLE "get "
#define GET_VARIABLE_EX "get @"  // get t0.txt, get j0.val, get "123", get 123
//1. When returned value is a string, the returned 
//  data is 0X70+ASCII code+0xff 0xff 0xff.
// 2. When returned value is numerical, returned 
// data is 0X71+4 byte binary data+0xff 0xff 0xff.
//The data storage mode is little-endian mode (namely, low-order in front, 
//and high-order at back).


#define GET_PAGE_ID "sendme"

#define CONVERT_VAR "cov "
#define CONVERT_VAR_EX "cov @,@,#"  // cov att1,att2,length
// cov h0.val,to.txt,0 //convert the value variable of slider h0 val into decimal 
//string and assign the txt variable of t0, the length is automatic
//cov t0.txt,j0.val,0  //convert the string variable of t0 txt into value and 
//assign the variable of slider h0 val, the length is automatic

#define CALIBRATE_TOUCH  "touch_j"  // enter calibrate function


#define SET_VISIBILITY "vis "
#define SET_VISIBILITY_EX "vis @,#"  // vis b0,1 == show, vis b0,0 == hide
#define SET_VISIBILITY_ID_EX "vis #,#" // vis 1,1 == show component ID 1, vis 1,0 == hide component ID 1

#define SET_TOUCH_ABILITY "tsw "
#define SET_TOUCH_ABILITY_EX "tsw @,#"  // tsw b0,1 == enable touch, tsw b0,0 == disable
#define SET_TOUCH_ABILITY_ID_EX "tsw #,#" // tsw 1,1 == enable touch component ID 1


#define COM_EXECUTION_STOP "com_stop"  // stop executing received commands
#define COM_EXECUTION_START "com_star"  // start executing received commands
#define EXECUTION_BUFFER_CLEAR "code_c"

#define RANDOM_INITIALIZE_EX "ranset #,#" // set value randomly generated from # to #

#define PRINT_RAW "print "
#define PRINT_RAW_EX "print @"   // print t0.txt  (string), print j0.val  (4 byte hex value)

#define PRINT_HEX "printh "


//-------------------------------------------
// Waveform components
//-------------------------------------------
#define ADD_DATA_TO_WAVEFORM_COMPONENT "add "
#define ADD_DATA_TO_WAVEFORM_COMPONENT_EX "add #, #, #" // data -- 0 to 255
//add 1, 0, 30  //add data 30 to channel 0 of the Waveform component which ID number is 1
//add 1, 1, 50  //add data 50 to channel 1 of the Waveform component which ID number is 1


#define ADD_DATA_TO_WAVEFORM_COMPONENT_IN_VOLUME "addt "
/*
addt: Add data to waveform component in volume

addt objid, ch, qty

objid: waveform component ID

ch: channel number of waveform component

qty: adopted point quantity of the data

    Example:

addt 1, 0, 100  //waveform component whose ID is 1 enter into data pass through mode, pass 
through adopted point quantity is 100

Remarks:

1.Waveform component only support 8-bit values, 0 minimum, 255 maximum. Single pass through 
is 1024 bits maximum.

2.After sending waveform data pass through command, it will take some time to get the device 
responded and started passing through values. This period takes about 5ms(it will take longer
if there are other commands to be executed in the buffer zone before executing data pass through 
command). Following this period, it will send a data pass through ready data(0XFE+Terminator) to 
user, then it will start sending pass through data. The data being passed are only hexadecimal 
numbers. It recovers to command receiving state only after the device has adopted specified data.

3.Waveform will not refresh until specified data has been passed through completely. 
*/

#define CLEAR_WAVEFORM_DATA_IN_COMPONENT "cle "
#define CLEAR_WAVEFORM_DATA_IN_COMPONENT_EX "cle #,#" //
//cle 1, 0   //clear the data of waveform component ID=1, channel=0;
//cle 1, 255   //clear all the data of waveform component ID=1 



//-------------------------------------------
// EEPROM
//-------------------------------------------

#define WRITE_EPROM "wepo "
#define WRITE_EPROM_FROM_COMPONENT "wepo @,#"
// wepo t0.txt,10 // write value of t0.txt to EEPROM, start from address #10. storage size is (t0.txt-mal)+1 bytes
#define WRITE_EPROM_FROM_STRING "wepo S,#" 
// wepo "abcd",10 // write string "abcd" to EEPROM, start from address #10. storage size is 5 bytes
//wepo 11,10  //write constant 11 to EEPROM, start from address #10. storage size 4 bytes

#define WRITE_EEPROM_FROM_UART "wept "
/*
wept: write data in hex to EEPROM through UART - Enhanced Model Only

wept add, length

add: start address in EEPROM

length: length of data


Example:

wept 30, 20 // write 20 bytes of hex data from UART to EERPOM, start address is 30 

Remark:

1, When wept instruction received, Nextion device will take around 5ms to initiate 
and send ready signal “0xFE 0xFF 0xFF 0xFF"

2, Before you send data from UART, you should wait until you get ready signal from 
Nextion device

3, wept can not be terminated. 
*/

#define READ_EEPROM_TO_UART "rept "
/*
rept: read data in hex from EEPROM to UART - Enhanced Model Only

rept add, lenght

add: start address in EEPROM

length: length of data

Example:

rept 30, 20 // read 20 bytes of hex data from EEPROM to UART, start address is 30 
*/

//-------------------------------------------------------
// GPIO
//-------------------------------------------------------
/*
cfgpio: configure GPIO - Enhanced Model Only

cfgpio id,state,cmp

id: I/O pin number

state: work state of the GPIO

  0 - pull up input mode
  1 - input binding mode
  2 - push pull output mode
  3 - PWM output mode
  4 - open drain output mode

cmp: binding component, available in work state 2

Example:

cfgpio 0, 0, 0    //Configure GPIO0 as pull up input mode, you can read the input status from system variable pio0, 
                       //such as: n0.val = pio0

cfgpio 1, 2, 0   //Configure GPIO1 as push pull output mode, you can control the output level 
                 //by system variable pio1, such as: pio1 = 1

cfgpio 2, 1, b0   //Configure GPIO2 as input binding mode, the binding component is button b0. 
                  //Falling edge of GPIO2 will trigger b0's button press event, rising edge of 
                  //GPIO2 will trigger bo's button release event.

cfgpio 4, 3, 0   //Configure GPIO4 as PWM output mode, you should use system variable pwm4 
                 //to set duty cycle before you use the instruction

Remark:

1, only GPIO4 to GPIO7 support PWM function

2, When configure a GPIO to input binding mode, only components in current page can be bind. It is recommended 
to put the instruction in the preinitialize event, because binding event wont not be triggered after page refreshing or switching. 

*/


//-------------------------------------------------------
// response table
//--------------------------------------------------------------
// all responses are terminated with 0xff 0xff 0xff
// for example, ivalid instruction returns 0x00 0xff 0xff 0xff
// Nextion responses
#define INVALID_INSTRUCTION     0x00
#define INSTRUCTION_EXECUTED    0x01
#define COMPONENT_ID_INVALID    0x02
#define PAGE_ID_INVALID         0x03
#define PICTURE_ID_INVALID      0x04
#define FONT_ID_INVALID         0x05
#define BAUD_RATE_SET_INVALID   0x11
#define CURVE_CTL_ID_INVALID    0x12
#define VAR_NAME_INVALID        0X1A
#define VAR_OPERATION_INVALID   0X1B
#define FAILED_TO_ASSIGN        0X1C
#define EEPROM_OPERATION_FAILED 0X1D
#define PARAMETER_QUANTITY_BAD  0X1E
#define UNDEFINED_ESCAPE_CHAR   0X20
#define VARIABLE_NAME_TOO_LONG  0X23  // MAXIMUM 29 CHARS


// Nextion EVENT Data 

#define TOUCH_EVENT             0x65
#define CURRENT_PAGE_ID         0x66
#define TOUCH_COORDINATES       0x67
#define TOUCHED_WHILE_ASLEEP    0x68
#define STRING_DATA             0x70
#define NUMBER_DATA             0x71
#define AUTO_ENTERED_INTO_SLEEP 0x86
#define AUTO_WOKE_UP            0x87
#define STARTUP_SUCCESSFUL      0x88
#define DATA_TRANSPARENT_TRANSMIT_FINISHED  0xFD
#define DATA_TRANSPARENT_TRANSMIT_READY     0xFE


// Color code list
#define COLOR_RED   63488
#define COLOR_BLUE  31
#define COLOR_GRAY  33840   
#define COLOR_BLACK 0
#define COLOR_WHITE 65535
#define COLOR_GREEN 2016
#define COLOR_BROWN 48192
#define COLOR_YELLOW 65504


/* [] END OF FILE */
