/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
/*
  Note: in "c" all hexadecimal numbers are preceded by 0x (or 0X) 
  Order of events:

  0) psoc comes up and puts out a message: "global.psRebooted.val=1" 0xff 0xff 0xff
  1) Nextion puts out a message: "|a0xff 0xff 0xff "
  2) psoc puts out a request for information: "global.SystemChange.val=1" 0xff 0xff 0xff

    note: all info sent to nextion over UART is terminated with 
      0xff 0xff 0xff ( 3 bytes all ones)

  todo add automatic/manual
*/
#include "main.h"
#include <stdio.h>
#include "playIR.h"

// IF USING SPRINTF, 
    //#if defined (__GNUC__)
// in Menu Build settings
// use default libraries true
// use newlibnano false
// use newlibnano floating point format false
    /* Add an explicit reference to the floating point printf library */
    /* to allow the usage of floating point conversion specifiers. */
    /* This is not linked in by default with the newlib-nano library. */
    // in CYDWR->System, change stack size to 4k (0x1000), and Heap size to 2k (0x7ff)
    //asm (".global _printf_float");
//#endif



#define millis milliseconds
extern int32  EncTicks;

enum captureTypesEnum manualHC=coolType;// default to cool

uint8 nextionWoke;
int16 returnAfterWaitStateState;

unsigned long waitStateStartTime;

int16 commandInWork;

enum nextionStates {
    nextionNoState=0
    ,nextionConfigure
    ,nextionSetPSOCAwake    
    ,nextionWaitmsec     // waitxxxmsec for message
    ,SYS_STARTUP_MSG
    ,SYS_REPORTING
    ,SYS_IDLE
};

enum nextionStates SystemState;

#define NUMBER_OF_IN_BUFS 4
int16 head,tail;
uint8 inBuff[NUMBER_OF_IN_BUFS][IN_BUFF_SZ];
uint8  outBuff[OUT_BUFF_SZ];
int16 bufIndex;
uint8 *curBuff;
int16 cbIndex;

float coolTemp=68.0f;
float heatTemp=72.0f;
int16 onOff=1; // if YES (1) is on.  control the aircon
int16 operatingMode; // if 1, heat, if 0 cool
int16 recordingFinished;
int16 manualMode ; // 1== manual, 0== automatic

// point to the isr
CY_ISR_PROTO(Nextion_rx_data);

//-----------------------------------------
//-----------------------------------------
void Nextion_interface_init(){         
    isr_UartRX_StartEx(Nextion_rx_data);
    UART_1_Start();// the NEXTION display communication interface
    UART_1_ClearTxBuffer();
    UART_1_ClearRxBuffer();

    SystemState = nextionConfigure;
    head=tail=0;
    curBuff = inBuff[head]; //toggle between buffers
}

//-----------------------------------------
// convert to ascii, return length
//-----------------------------------------
int itoa(int number, unsigned char* buf)
{
	int sum = number;
	int i = 0;
	int digit;
    int negative=0;
    
    if (sum < 0 ) {
        negative = 1;
        sum=-number;
    }
    
	do
	{
		digit = sum % 10;
		if (digit < 0xA)
			buf[i++] = '0' + digit;
		else
			buf[i++] = ('A'- 0xA) + digit ;
		sum /= 10;
	}while (sum);
    
    if (negative){
        buf[i++]='-';
    }
	
	buf[i] = '\0';
	return i;
}
//-----------------------------------------
// A short atoi() function
//-----------------------------------------
int atoi(char *str)
{
    int result = 0;     // Initialize result
    int16 sign = 1;     // Initialize sign as positive
    int16 index = 0;    // Initialize index of first digit or sign
      
    // If number is negative, then update sign
    if (str[0] == '-'){
        sign = -1;  
        index++;  // move past sign
    }
      
    // go left to right, multiply by 10 each time
    // will screw up on anything over 0x39...
    // will stop on 0x0d, 0x0a, or anything
    // less than ascii 0.
    for (; str[index] && str[index]>0x2f; ++index)
        result = result*10 + str[index] - '0';
    
    // Return result with sign
    return sign*result;
}


uint8 swallowFFs; // when true, we must clear out FF's before looking for other data
uint8 zeroDepth;

//--------------------------------------------------
enum psocCmds {
    REQUEST_CONFIG= 0
    ,PSOC_REBOOTED 
    ,PSOC_AWAKE
    ,INC_MSG_COUNTER
    ,DEBUG_TXT_MSG
};
//--------------------------------------------------
char *NextionCommands[] = {
    "global.SystemChange.val=1"
    ,"global.psRebooted.val=1"
    ,"global.psoc_awake.val=1"
    ,"global.PsocMsgCounter.val++"
    ,"main.tDebugText.txt="
};
//--------------------------------------------------
//--------------------------------------------------
void sendTextToNextion(char *text){
    // make a copy of the text in case 
    // this function is interrupted.
    strcpy((char*)outBuff,text);
    char *p=(char*)outBuff;
    while (*p) {
        UART_1_PutChar(*p++);
    }
    // write command terminator
    UART_1_PutChar(0xff);
    UART_1_PutChar(0xff);
    UART_1_PutChar(0xff);
    
}
//--------------------------------------------------
// set countdown text variable in main screen
//--------------------------------------------------
void sendCountDownTextToNextion(int16 count){
    static char n[4]={'\"',0x30,'\"',0};  
    n[1]=(count&0xf)+0x30;   
    strcpy((char*)outBuff,"main.tCountDown.txt=");
    strcat((char*)outBuff,n);
    sendTextToNextion((char*)outBuff);
}
//--------------------------------------------------
//--------------------------------------------------
void sendDebugTextToNextion(char *msg){
    static char q[2]="\"";
    strcpy((char*)outBuff,NextionCommands[DEBUG_TXT_MSG]);
    strcat((void*)outBuff,q);
    strcat((void*)outBuff,msg);
    strcat((void*)outBuff,q);
    sendTextToNextion((void*)outBuff);
}

//--------------------------------------------------
//--------------------------------------------------
void sendCommandToNextion(int16 CommandNumber){
    sendTextToNextion(NextionCommands[CommandNumber]);
}

//-----------------------------------------
// update the main display temperature
// text field.
//-----------------------------------------
void sendTemperatureToNextion() {
    extern float temperature;

    sprintf((char*)outBuff,"main.curTemp.txt=\"%3.1f\"",temperature);
    sendTextToNextion((char*)outBuff);
}
//--------------------------------------------------
// Record UP    "|ru"
// Record DOWN  "|rd"
//--------------------------------------------------
void handleRecordCommands(uint8 *bufPtr){

    switch(bufPtr[1]){
        case 'u':
            captureType = upType;
        break;
        
        case 'd':
            captureType = downType;
        break;
        
        case 's':
                recordingFinished=TRUE;
        break;
        
        default:
         return;
    }
    
}
//--------------------------------------------------
// temperature set commands
// Cool Temp    "|tc"  -- followed by temp i.e. "ct72"
// Heat Temp    "|th"  -- followed by temp i.e. "ht72"
// the "|" is removed in the serial stream
//--------------------------------------------------
void handleTempSetCommands(uint8 *bufPtr){
 //   (void)bufPtr;
    switch(bufPtr[1]){
        case 'c': {// set cool temperature
           int16 ct = atoi((char*)&bufPtr[2]);
            coolTemp = (float)ct;
        }break;
            
        case 'h':{
          int16 ht   = atoi((char*)&bufPtr[2]);
            heatTemp=(float)ht;
        }break;
        default:
            break;
    }
}
//--------------------------------------------------
//--------------------------------------------------
void handleOnOffIR(uint8 *bufPtr){
      switch(bufPtr[1]){
        case '0': // set on/off status
            onOff = atoi((char*)&bufPtr[2]);
        break;
            
        case '1':// 0== cool 1== heat
            operatingMode = atoi((char*)&bufPtr[2]);
        break;
        default:
            break;
    }
}
//--------------------------------------------------
//--------------------------------------------------
void handleOpStateCommands(uint8 *bufPtr){
      switch(bufPtr[1]){
        case 'o': // set on/off status
            onOff = atoi((char*)&bufPtr[2]);
        break;
            
        case 'm':// 0== cool 1== heat
            operatingMode = atoi((char*)&bufPtr[2]);
        break;
        default:
            break;
    }
}
//--------------------------------------------------
//   fan up     "|fu"  // cycle through fan settings
//   fan down   "|fd"
//   fan auto   "|fa"
//--------------------------------------------------
void handleFanChoice(uint8 *bufPtr){
     switch(bufPtr[1]){
        case 'u': // fan faster
            addPlayIRToQueue(fanFasterType,CurrentEquipmentID);
        break;
            
        case 'd':
            addPlayIRToQueue(fanSlowerType,CurrentEquipmentID);
        break;
        
        case 'a':
            addPlayIRToQueue(fanAutoType,CurrentEquipmentID);
        break;            
        default:
            break;
    }
}
//--------------------------------------------------
//--------------------------------------------------
void handleSendIRByCmd(uint8 *bufPtr){
    switch(bufPtr[1]){
        case 'u': // play up
            addPlayIRToQueue(upType,CurrentEquipmentID);
        break;
            
        case 'd':// play down
            addPlayIRToQueue(downType,CurrentEquipmentID);
        break;
            
        case 'e': // energy saver
            addPlayIRToQueue(energySaverType,CurrentEquipmentID);
        break;
            
        case 'c': // cool (forced by choosing this)
            addPlayIRToQueue(coolType,CurrentEquipmentID);
        break;
            
            
        default:
            break;
    }
}
//--------------------------------------------------
//--------------------------------------------------
void handleSetEquipmentType(uint8 *bufPtr){
    switch(bufPtr[1]){
        case 't': // set type
            CurrentEquipmentID = (enum equipEnum) atoi((char*)&bufPtr[2]);
        break;
            
        default:
            break;
    }
}
//--------------------------------------------------
// send commands.  Note that the addPlayIRToQueue()
// system will space the IR commands appropriately
//---
// Manual
//   manual on  "|m1"
//   manual off "|m0"
//    temp up   "|mu"
//    temp down "|md"
//    heat      "|mh"
//    cool      "|mc"
// energy saver "|me"
//---
// GE Commands
//   select     "|mm"  rotate through the selections
//--------------------------------------------------
void handleManualCommands(uint8 *bufPtr){
    //int16 waitCount=0;
    switch(bufPtr[1]){
        case 'u':  // temperature change
            addPlayIRToQueue(upType,CurrentEquipmentID);
        break;
            
        case 'd':
            addPlayIRToQueue(downType,CurrentEquipmentID);
        break;
            
            
        case 'h': // mode select
            manualHC=heatType;
            addPlayIRToQueue(manualHC,CurrentEquipmentID);
        break;
            
        case 'c': // mode select
            manualHC=coolType;
            addPlayIRToQueue(manualHC,CurrentEquipmentID);
        break;
            
        case 'e': 
            manualHC=energySaverType;
            addPlayIRToQueue(manualHC,CurrentEquipmentID);

        break;
            
        case '0':// manual mode being turned off
            manualMode=0;// turn on temperature control
        break;
            
        case '1':// manual mode on
            manualMode=1;// turn off temperature control
        break;
            // GE Equipment Mode select
        case 'm': // manual mode "Select" "Next" operating mode
            addPlayIRToQueue(modeType,CurrentEquipmentID);
        break;
        
            
        default:
            break;
    }
}
            
//--------------------------------------------------
// commands to psoc
// Nextion Awake "|a"
// Set Equipment "|et"
//---
// Manual
//   manual on  "|m1"
//   manual off "|m0"
//    temp up   "|mu"
//    temp down "|md"
//    heat      "|mh"
//    cool      "|mc"
// energy saver "|me"
// following permanent choice
//   fan up     "|fu"  // cycle through fan settings
//   fan down   "|fd"
//   fan auto   "|fa"
//---
// Power Toggle "|pt"
// Record COOL  "|rc"
// Record HEAT  "|rh"
// stop Record  "|rs"
// Record UP    "|ru"
// Record DOWN  "|rd"
// On State     "|S1"
// Off State    "|S0"
// Cool Temp    "|tc"  -- followed by temp i.e. "ct72"
// Heat Temp    "|th"  -- followed by temp i.e. "ht72"
// OnOff        "|oo"  -- followed by "1" (on) or "0" (off)
// Operate Mode "|om" - followed by "0" (cool) or "1" (heat)
//----------
// commands from psoc
// curTemp.txt="72.5" 0xff 0xff 0xff
// main.psoc_awake.val=1 0xff 0xff 0xff
//
// SLEEP:
//  AUTO SLEEP STARTED: 0x86 0xff 0xff 0xff
//       WAKUP        : 0x87 0xff 0xff 0xff
//--------------------------------------------------
void handle_Nextion_interface() {
    
    
    
    while (head != tail ){
        
        curBuff=inBuff[tail];
        switch(curBuff[0]){
            case '1':
            // received nextion awake
            case 'a':{
                sendCommandToNextion(PSOC_AWAKE);
                CyDelay(10); // wait 10 milliseconds
                sendCommandToNextion(REQUEST_CONFIG);
                nextionWoke=1;// start reporting state machine.
            }break;
            case 'e':{
                CyDelay(2);
                handleSetEquipmentType(curBuff);
            }break;
            
            case 'f':{
                CyDelay(2);
                handleFanChoice(curBuff);
            }break;
            case 'm':{
               // manualMode=1;
                CyDelay(2);// allow the rest to come in
                // handle manual command
                handleManualCommands(curBuff);
            }break;
            case 'o':{
                // handle operational state commands
                handleOpStateCommands(curBuff);
            }break;
            case 'p':{ // Power toggle
                 CyDelay(2);// allow the rest to come in
                 if (curBuff[1]=='t'){
                    addPlayIRToQueue(powerType,CurrentEquipmentID);
                }
            } break;
            // received record command
            case 'r':{
                // these are "record" commands
                //  the user will point to the receiver and
                // press the appropriate key
                CyDelay(2);// allow the rest to come in
                 handleRecordCommands(curBuff);
            }break;
            case 's':{
                // handle OnOff IR command.
                CyDelay(2);// allow the rest to come in
                 handleOnOffIR(curBuff);
            }break;
            case 't':{
                // temperature set commands
                CyDelay(2);// allow the rest to come in
                 handleTempSetCommands(curBuff);
                decisionTimer=0;// update a/c unit
            }break;     
            
            case 'u':{
                // auto wakeup.
                msCounter1=10; // take temperature and update display
                decisionTimer=0;// update a/c unit
            }break;
            
        }
        
        if (++tail >= NUMBER_OF_IN_BUFS)
            tail=0;
    }
    
    
    // if nextion rebooted, we may have to reconfigure.
    // so go through process again.
    if (nextionWoke==1)
        SystemState=SYS_STARTUP_MSG;
   
    
    switch(SystemState)
    {
    /*****************/
        // once per second update the current temperature
    case SYS_REPORTING:{
        SystemState = nextionWaitmsec;
        returnAfterWaitStateState = SYS_REPORTING;
        waitStateStartTime = millis +1000;
        
        if (nextionWoke){
            // if a reading is there, give to psoc
            extern int16 TemperatureReadingAvailable;
            if (TemperatureReadingAvailable){
                sendTemperatureToNextion();
                TemperatureReadingAvailable=FALSE;
                sendCommandToNextion(INC_MSG_COUNTER);
                
            }
            
        }else{
            sendCommandToNextion(PSOC_REBOOTED);            
            sendCommandToNextion(INC_MSG_COUNTER);
            sendDebugTextToNextion("PSOC Awake");

        }
        
	}break;
        
    /*******************/
    case SYS_STARTUP_MSG:
        if (nextionWoke==1) {
    		SystemState = nextionConfigure;
            nextionWoke=2; // set to next state, so we don't keep returning here
        }else {
            SystemState = nextionWaitmsec;
            returnAfterWaitStateState = SYS_STARTUP_MSG;
            waitStateStartTime = millis +1000;    // wait 1 second            
        }
    break;
        
    case nextionConfigure: {
       // nextionSendPsocIsAwake();
        SystemState = nextionWaitmsec;
        returnAfterWaitStateState = SYS_REPORTING;
        waitStateStartTime = millis +100;  // wait 1/10th second
      }break;
        
    case nextionWaitmsec:{            
        if (millis >= waitStateStartTime) 
            SystemState = returnAfterWaitStateState;
    }break;
        
    /********************/
    case SYS_IDLE:
    default:
        
    break;
    }

}
//------------------------------------------------------
// the interrupt for RS232 data from the Nextion
//------------------------------------------------------
 CY_ISR(Nextion_rx_data){
     uint8 temp ;
    static int16 BeginFound;
    
    do {
        temp = UART_1_GetChar();
        
        // by convention, our nextion code
        // does not send binary numbers, only 
        // ascii numbers, so we can safely ignore 0xff.
        if (temp==0xff)
            continue;
        if (temp=='|'){
            BeginFound=YES;
            cbIndex=0;
            continue;
        }
        
        if (temp==0x87){ // auto wakeup
            // trigger the psoc to send wakeup 
            BeginFound=YES;
            cbIndex=0;
            temp=(uint8)'u'; // trigger auto wakeup.
            inBuff[head][cbIndex++]=temp;
            temp=0xa;// trigger command 
        }
        
        if (!BeginFound)
            continue; // throw away garbage
        
        inBuff[head][cbIndex++]=temp;
        // we ask the nextion to send an '\r' in the text, so it
        // also sends an '\n'.  key on that.
        if (temp==0xa){            
            if (++head >= NUMBER_OF_IN_BUFS)
                head=0;            
            cbIndex=0;
        }
             
        if (cbIndex >= IN_BUFF_SZ)
            cbIndex=0; // something bad is happening, clear the rx buffer pointer
    } while (UART_1_GetRxBufferSize());
    
}



/* [] END OF FILE */
