/* ========================================
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 *  Under the GPL v3 license
 * This license does not override previous licenses
 * Some information may be Proprietary to 
 * Cypress (http://www.cypress.com) for their
 * PSoC 5LP®--Cypress Semiconductor and
 * only usable on their devices.
 * PROPERTY OF Wade Maxfield.
 * Commercial license available
 * ========================================
*/
#include "parseCommands.h"
#include <stdio.h>
#include "USBSerial.h"
#include "captureIR.h"
#include "atoi_itoa.h"
#include "playIR.h"

char outMsg[64];
char commandBuffer[64];
int16 commandBufferIndex;

int16 debugPrint;

//-------------------------------------------
//-------------------------------------------
void init_parse() {
    uint16 c=1;
    parse("v",&c);// show sign on message
}

int16 handleDoCapture(){

    
        //extern int16 captureIRData;
        switch(commandBuffer[1]){
            case 'T': case 't':
                printMessage("\n\rNow press IR button for function capture ");

                switch (commandBuffer[2]){
                    case 'u':case'U':
                        captureType=upType;
                        printMessage("of UP button.\n\r");
                    break;
                    case 'd':case'D':
                        captureType=downType;
                        printMessage("of DOWN button.\n\r");
                    break;
                    case 'P':case'p':
                        captureType=powerType;
                        printMessage("of POWER button.\n\r");
                    break;
                        
                    case 'h':case 'H':
                        captureType=heatType;
                        printMessage("of HEAT select button.\n\r");
                    break;
                        
                    case 'c':case 'C':
                        captureType=heatType;
                        printMessage("of COOL select button.\n\r");
                    break;
                        
                    case 'f': case 'F':
                        captureType=fanOnlyType;
                    break;
                        
                    case 'm': case 'M':
                         captureType=modeType;
                    break;
                        
                    default:
                        printMessage("\n\rCommand not understood\n\r");
                        return 1;
                    } 

                                    
                printMessage("Hit Enter to stop capture.>");
                captureIRData=1;
            return 0;// don't print signon message

            
                
            case 'd': case 'D':
                return 2;          
                
            default:
                 printMessage("\n\rCommand Not Understood\n\r");
                return 1;// print signon

            }// switch commandbuffer[1]

}
           
int16 handlePlayIR() {
    
    // play 
    printMessage("For Equipment: ");
    if (!CurrentEquipmentID)
       printMessage("Fridgidaire\n\r");
    else 
        printMessage("GE" "\n\r");

    printMessage("Playing IR sequence ");
    switch (commandBuffer[1]){
        case 'u':case'U':
            captureType=upType;
            printMessage("of UP button.\n\r");
        break;
        case 'd':case'D':
            captureType=downType;
            printMessage("of DOWN button.\n\r");
        break;
        case 'P':case'p':
            captureType=powerType;
            printMessage("of POWER button.\n\r");
        break;
            
        case 'h':case 'H':
            if (CurrentEquipmentID==GE_ID){
                printMessage("Command Not Available for GE\n\r");
                //goto printSignon;
                return 1;
            }
            captureType=heatType;
            printMessage("of HEAT select button.\n\r");
        break;
            
        case 'm': case 'M':
            if (CurrentEquipmentID==FRIDGIDAIRE_ID){
                printMessage("MODE Command Not Available for Frigidaire\n\r");
                //goto printSignon;
                return 1;
            }
            captureType=modeType;
            printMessage("of MODE select button.\n\r");
        break;
            
            
        case 'c':case 'C':
            if (CurrentEquipmentID==GE_ID){
                printMessage("Command Not Available for GE\n\r");
               // goto printSignon;
                return 1;
            }
            captureType=coolType;
            printMessage("of COOL select button.\n\r");
        break;
        
        default:
            printMessage("\n\rCommand not understood\n\r");
           // goto printSignon;
            return 1;
    }
    
    playIRSignal(captureType,CurrentEquipmentID);
    return 1;
}

    
void PrintDump() {
    int32 result;
    int16 index,index1;
    
    sprintf(outMsg,"Captured %d readings\n\r",capCounter);
    printMessage(outMsg);
    index1=0;
    for (index =0; index < capCounter; index++){
        result = SendTimes[index];
        sprintf(outMsg,"%2d : %5ld ",index,result);
        if (++index1>9){
            index1=0;
            printMessage("\n\r");
        }
        printMessage(outMsg);
    }
    
    printMessage("\n\rabsolute:\n\r-----\n\r");
    index1=0;
    for (index =0; index < capCounter; index++){
        result = CaptureTimes[index];
        sprintf(outMsg,"%2d : %5ld ",index,result);
        if (++index1>5){
            index1=0;
            printMessage("\n\r");
        }
        printMessage(outMsg);
    }
    

}
    

//-------------------------------------------
//-------------------------------------------
void parse(char *RXBuffer, uint16 *rxCountp) {
   // static int16 unlock;
    int16 rxCount = *rxCountp;
    static int16 bufferIndex;    
    *rxCountp=0;// reset indicating we consumed buffer
    char c;
    int index,index1;
    int32 result;
                        
    
    
    for (bufferIndex =0 ; bufferIndex < rxCount; bufferIndex++) {
        c=commandBuffer[commandBufferIndex++]=RXBuffer[bufferIndex];
        printChar(c);
        if (c != '\r')
            continue;
        
        printChar('\n');
        commandBufferIndex=0;
        // we have received data from usb.  look at it and determine what to do
        switch(commandBuffer[0]) {
            case 0:
            case '\n':
            case '\r':{
                uint16 result;
                int16 index;
                if (captureIRData==2){
                    captureIRData=3;
                    printMessage("\n\rCapture stopped.\n\r");
                    // save the calculated send time values here.
                    for (index =1; index <= capCounter; index++){
                        result = CaptureTimes[index]-CaptureTimes[index-1];
                        SendTimes[index-1]=result;
                    }
                    PrintDump();
                }
            }
            default:                               
            case 'v':
            case 'V':
            printSignon:
                printMessage("\n\n\r--------\n\r");
                printMessage(VERSION_STRING);
                printMessage("commands: (press enter after command to execute)\n\r---------\n\r");
                printMessage(" CT[]-- Capture Type []==U ,    D,     P,    H,   C,    F     , M (for GE) \n\r"
                             "                        (up, down, power, heat, cool, fan only, Mode)\n\r");
                printMessage(" CD     -- Dump Capture Readings\n\r");
                printMessage(" D      -- Toggle Debug Print\n\r");
                printMessage(" F      -- Formatted Dump of Capture\n\r");
                printMessage(" P[]    -- Play IR []==U,D,P,H,C (up, down, power, heat, cool)\n\r");
                printMessage(" S[]    -- Select equipment 0=FR,1=GE, default 0\n\r");
                printMessage(" T      -- Show Temperature\n\r");
                printMessage(" V      -- Version / help\n\r");
                printMessage("--\n\renter command>");
                
                return;
                
            case 'c':
            case 'C':{ 
                switch(handleDoCapture()){
                   case 2:
                        PrintDump();
                    case 1:
                        goto printSignon;
                         
                    default:
                        break;
                }
            }break;
                
            case 'p': case 'P':
                if (handlePlayIR())
                    goto printSignon;
                
            break;
            
            case 's': case 'S':{
                // select equip
                if (commandBuffer[1]=='1'){
                    CurrentEquipmentID = GE_ID;
                    printMessage("GE chosen\n\r");
                } else {
                    CurrentEquipmentID = FRIDGIDAIRE_ID;
                    printMessage("Fridgidaire chosen\n\r");
                }
                goto printSignon;
                
            }break;
            case 't': case 'T':{
                extern float temperature;
                printMessage("\n\rTemperature: ");
                sprintf(outMsg,"%5.2f\n\r",temperature);
                printMessage(outMsg);
                printMessage("--\n\renter command>");
                return;        
            } 
            
            case 'f': case 'F':{
                sprintf(outMsg,"Captured %d readings\n\r{ ",capCounter);
                printMessage(outMsg);
                index1=0;
                for (index =0; index < capCounter; index++){
                    result = SendTimes[index];
                    sprintf(outMsg," %5ld",result);
                        
                    if (++index1>9){
                        index1=0;
                        printMessage("\n\r");
                    }
                    printMessage(outMsg);
                    if (index < capCounter-1)
                        printMessage(", ");
                }
              //  printMessage(",  0");// terminate array
                printMessage("\n\r};\n\r");
                    
            } goto printSignon;
            
            case 'd': case 'D':
                debugPrint=!debugPrint;
                printMessage("Debug print now ");
                if (debugPrint)
                    printMessage("on\n\r");
                else
                    printMessage("off\n\r");
                    
                    goto printSignon;
            break;
            }//switch(commandBuffer
    }//for()
  //seqNotFound:
    return;
    
}


/* [] END OF FILE */
